/* pxy.h: open proxy checker, common definitions.
 * Michael Tokarev  <mjt@corpit.ru>.
 * This code may be freely used and distributed according to
 * the terms of General Public License (GPL) version 2 or later.
 */

#ifndef _PXY_H
#define _PXY_H

typedef struct pxyconn pxyconn_t;
typedef struct pxyproto pxyproto_t;

typedef unsigned short ipport_t;

typedef struct {
  unsigned len;	/* current length in buffer */
  char buf[128-3*sizeof(unsigned)]; /* the buffer itself */
} pxybuf_t;

extern unsigned char pxybuf[8193];

/* Proxy protocol definition. */
struct pxyproto {
  const char *name;	/* canonical name of a protocol */
  const char *aname;	/* alternative name */
  int advanced;		/* true if this proto should NOT be used by default */
  const ipport_t *defports; /* 0-terminated array of default ports */
  void (*handler)(int fd, ev_t e, pxyconn_t *c); /* protocol handler */
  int (*check)(int fd, pxyconn_t *c, char *buf, int l);
};

extern const pxyproto_t pxyprotos[]; /* array of all supported protocols */
const pxyproto_t *pxyfindproto(const char *name);

/* Active proxy connection structure.
 * There aren't many of those: upper limit is the
 * number of open files.
 * The structure supposed to be "embedded" into another
 * structure with additional data for higher-level protocol
 * handled by hlcbck routine.
 */
struct pxyconn {
  struct in_addr pxyaddr;	/* address of a proxy */
  struct in_addr dstaddr;	/* address of destination */
  ipport_t pxyport;		/* port number of a proxy */
  ipport_t dstport;		/* port number of destination */
  const pxyproto_t *proto;	/* proxy protocol description */
  pxybuf_t *buf;		/* buffer holding data read from net */
  int state;			/* current state */
  int nread;			/* number of bytes read so far */
  void *data;			/* app-supplied data */
};

extern int pxytimeout;

/* almost all routines returns >= 0 on ok (0 in case when value
 * is meaningless, or e.g number of bytes read) and <0 on error;
 * in error case, pxyaction will be called automatically. */

/* allocate new connection structure */
pxyconn_t *pxynew();

/* free connection resources, timers etc */
void pxyfree(int fd, pxyconn_t *c);

/* start new connection attempt */
int pxystart(int fd, pxyconn_t *c);

/* renew a timer; if tmo==0, use pxytimeout */
int pxyrenew(int fd, pxyconn_t *c, int tmo);

/* check for error condition, on error call pxyaction and return -1 */
int pxycerr(int fd, ev_t e, pxyconn_t *c);

/* read bytes from connection, return number of bytes read (>0)
   or call pxyaction and return 0 */
int pxyread(int fd, pxyconn_t *c, char *buf, int l, int loglevel);

/* read next chunk of bytes into pxybuf[] together with saved data,
   return number of bytes available (>0) or call pxyaction and
   return 0 */
int pxyreadnext(int fd, pxyconn_t *c, int loglevel);

/* save data from buf of len `len', at most max bytes, within
   connection structure.  Calls pxyaction on error */
int pxysave(int fd, pxyconn_t *c, char *buf, unsigned len, unsigned max);

/* write something from buf of length len to the remote, with logging */
int pxywrite(int fd, pxyconn_t *c, const char *buf, int len, int loglevel);

/* app-supplied: */
void pxyinfo(const pxyconn_t *c, int level, const char *fmt, ...);
void pxyaction(int fd, pxyconn_t *c, int result);
 /* 0 - connected, 1 - done, 2 - definitely not open, -1 - err.
  * Should free connection in case result != 0, or change event
  * callback if == 0. */

/* log the raw io: direction is 0 for read and 1 for write */
void pxyvio(pxyconn_t *c, int loglevel, int direction,
	    const char *buf, int bufl);

#if !defined(__GNUC__) && !defined(__attribute__)
# define __attribute__(c)
#endif
#ifndef UNUSED
# define UNUSED __attribute__((unused))
#endif

#endif
